/*
	$Id: Class.java 264 2006-08-18 21:19:31Z syl $

	Copyright (C) 2006 Syllabub
	Maroontress Fast Software.
*/

import java.io.File;
import java.io.PrintWriter;
import org.w3c.dom.Element;

/**
   饹Ǥ
*/
class Class extends Group {

    /** 󥯥롼ɤΥå */
    private Includes includes;

    /** ݡȤΥå */
    private Imports imports;

    /** ݡȤΥå */
    private Exports exports;

    /** ¤ΡΡ󷿤Υå */
    private Compounds compounds;

    /** ᥽åɤΥå */
    private RegularMethods methods;

    /** ֥ȥ饯ȥ᥽åɤΥå */
    private AbstractMethods abstractMethods;

    /** ޥΥå */
    private Macros macros;

    /** Υå */
    private Typedefs typedefs;

    /** ХѿΥå */
    private GlobalVariables globalVariables;

    /**
       饹ޤ

       @param elem class
    */
    public Class(final Element elem) {
	super(elem);

	ChildrenFilter c = getChildrenFilter();
	GroupManager gm = getGroupManager();
        includes = new Includes(c.getDirectChildrenByTagName("include"));

	imports = new Imports(gm, c.getDirectChildrenByTagName("import"));
	exports = new Exports(gm, c.getDirectChildrenByTagName("export"));
	methods = new RegularMethods(
	    gm, c.getDirectChildrenByTagName("method"));
	compounds = new Compounds(
	    gm, c.getDirectChildrenByTagName("struct|union|enum"));
	abstractMethods = new AbstractMethods(
	    gm, c.getDirectChildrenByTagName("abstractmethod"));
	macros = new Macros(gm, c.getDirectChildrenByTagName("define"));
	typedefs = new Typedefs(gm, c.getDirectChildrenByTagName("typedef"));
	globalVariables = new GlobalVariables(
	    gm, c.getDirectChildrenByTagName("globalvariable"));
    }

    /**
       {@inheritDoc}
    */
    @Override public void printCode() {
	String packageName = getPackageName();
	String name = getName();
	String dirName = packageName.replace('_', File.separatorChar);
	PrintWriter wr = createPrintWriter(dirName, name + ".h");
	String includeMacro = packageName + "_" + name + "_H";
	String importMacro = packageName + "_" + name + "_IMPORT";

	wr.println("#ifndef " + includeMacro);
	wr.println("#define " + includeMacro);

	includes.printCode(wr);
	macros.printCode(wr);
	imports.printCode(wr);
	exports.printCode(wr);
	typedefs.printCode(wr);
	compounds.printCode(wr);
	methods.printCode(wr);
	globalVariables.printCode(wr);

	if (isNamespaceExpanded()) {
	    wr.println("");
	    wr.println("#ifdef " + importMacro);
	    getGroupManager().printImportMacro(wr);
	    wr.println("#endif /* " + importMacro + " */");
	}

	wr.println("");
	wr.println("#endif /* !" + includeMacro + " */");
	wr.close();
    }

    /**
       {@inheritDoc}
    */
    @Override protected String getType() {
	return "class";
    }

    /**
       {@inheritDoc}
    */
    @Override protected void printHtml(final PrintWriter wr) {
	macros.printSummary(wr);
	typedefs.printSummary(wr);
	compounds.printSummary(wr);
	methods.printSummary(wr);
	abstractMethods.printSummary(wr);
	globalVariables.printSummary(wr);

	macros.printDetail(wr);
	typedefs.printDetail(wr);
	compounds.printDetail(wr);
	methods.printDetail(wr);
	abstractMethods.printDetail(wr);
	globalVariables.printDetail(wr);
    }

    /**
       {@inheritDoc}
    */
    @Override protected void printTex(final PrintWriter wr) {
	macros.printTexDetail(wr);
	typedefs.printTexDetail(wr);
	compounds.printTexDetail(wr);
	methods.printTexDetail(wr);
	abstractMethods.printTexDetail(wr);
	globalVariables.printTexDetail(wr);
    }

    /**
       {@inheritDoc}
    */
    @Override public void printStub() {
	printStubHeader();
	printStubCode();
    }

    /**
       ֤Υإåեޤ
    */
    private void printStubHeader() {
	String packageName = "stub_" + getPackageName();
	String name = getName();
	String dirName = packageName.replace('_', File.separatorChar);
	PrintWriter wr = createPrintWriter(dirName, name + ".h");
	String includeMacro = packageName + "_" + name + "_H";

	wr.println("#ifndef " + includeMacro);
	wr.println("#define " + includeMacro);

	String s = getPackageName() + "_" + getName();
	s = s.replace('_', File.separatorChar);

	wr.println("");
	wr.println("#include <" + s + ".h>");

	methods.printStubHeader(wr);
	globalVariables.printStubHeader(wr);

	wr.println("");
	wr.println("#endif /* !" + includeMacro + " */");
	wr.close();
    }

    /**
       ֤Υɤޤ
    */
    private void printStubCode() {
	String packageName = "stub_" + getPackageName();
	String name = getName();
	String dirName = packageName.replace('_', File.separatorChar);
	PrintWriter wr = createPrintWriter(dirName, name + ".c");

	String s = getPackageName() + "_" + getName();
	s = s.replace('_', File.separatorChar);

	wr.println("#include <stdlib.h>");
	wr.println("#include <stdarg.h>");
	wr.println("#include <" + s + ".h>");

	methods.printStubCode(wr);
	globalVariables.printStubCode(wr);
	wr.close();
    }
}
