/*
	$Id: IDManager.java 267 2006-08-19 16:33:30Z syl $

	Copyright (C) 2006 Syllabub
	Maroontress Fast Software.
*/

import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.w3c.dom.Element;

/**
   ̻ҥޥ͡Ǥ
*/
public final class IDManager {

    /** ͣΥ󥹥 */
    private static IDManager theInstance;

    /** splintȤɽ */
    private static final String REGEX_C_COMMENT = "/\\*@(([^/]|[^*]/)*)@\\*/";

    /** ѥɤɽ */
    private static final String REGEX_1 = "((struct|union|enum) @[1-9][0-9]*)";

    /** ѥ.Фɽ */
    private static final String REGEX_2
	= "((struct|union|enum) *)?[_A-Za-z]\\w*(\\.[_A-Za-z]\\w*)?";

    /** ưŪ˥󥯤оݤȤʤʸ */
    private static final String REGEX = REGEX_1 + "|" + REGEX_2;

    /** ֤̾ȥɤΥϥåޥå */
    private HashMap<String, Location> dictionary;

    /** REGEX򥳥ѥ뤷ɽΥѥ */
    private Pattern pattern;

    /** ǸΥˡID */
    private int num;

    /**
       @return ̻ҥޥ͡
    */
    public static IDManager getInstance() {
	if (theInstance == null) {
	    theInstance = new IDManager();
	}
	return theInstance;
    }

    /**
       ̻ҥޥ͡ޤ
    */
    private IDManager() {
	dictionary = new HashMap<String, Location>();
	pattern = Pattern.compile(REGEX);
	num = 0;
    }

    /**
       @return ˡID
    */
    public int getUniqueID() {
	return ++num;
    }

    /**
       ꤷ֤̾˥ɤϢդޤ

       @param location ֤̾ΰ
       @param keyword 
    */
    public void addKeyword(final Location location, final String keyword) {
	if (keyword.equals("")) {
	    return;
	}
	dictionary.put(keyword, location);
    }

    /**
       HTMLΥ󥫡ǽʸޤ

       @param s ʸ
       @return HTMLʸ
    */
    private String decoratedString(final String s) {
	Location location = dictionary.get(s);
	if (location == null) {
	    return s;
	}
	return location.decoratedString(s);
    }

    /**
       HTMLʸޤ

       @param s ʸ
       @return HTMLʸ
    */
    public String getString(final String s) {
	String in = s;
	String out = "";

	while (in.length() > 0) {
	    Matcher m = pattern.matcher(in);
	    if (!m.find()) {
		out += in;
		break;
	    }
	    int start = m.start();
	    int end = m.end();
	    out += in.substring(0, start);
	    out += decoratedString(in.substring(start, end));
	    in = in.substring(end);
	}
	return out.replaceAll(REGEX_C_COMMENT,
			      "<span class=\"splint\">($1)</span>");
    }

    /**
       ʸCμ̻ҤƬʸȤڤޤ

       @param c ʸ
       @return trueǤʤfalse
    */
    private boolean isPrimaryChar(final char c) {
	return (c == '_' || (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z'));
    }

    /**
       ʸCμ̻Ҥ2ʸܰʹߤʸȤڤޤ

       @param c ʸ
       @return trueǤʤfalse
    */
    private boolean isSecondaryChar(final char c) {
	return (isPrimaryChar(c) || (c >= '0' && c <= '9'));
    }

    /**
       ʸCμ̻ҤȤڤޤ

       @param s ʸ
       @return trueǤʤfalse
    */
    public boolean isCID(final String s) {
	if (!isPrimaryChar(s.charAt(0))) {
	    return false;
	}
	for (int k = 1, len = s.length(); k < len; ++k) {
	    if (!isSecondaryChar(s.charAt(k))) {
		return false;
	    }
	}
	return true;
    }

    /**
       ǤCμ̻ҤȤʤ°ͤޤ

       ͤʸξNodeException򥹥ޤ

       @param elem 
       @param attrName °̾
       @return °
    */
    public String getIDAttribute(final Element elem, final String attrName) {
	String s = elem.getAttribute(attrName);
	if (s.equals("")) {
	    String m = attrName + " attribute is empty.";
	    throw new NodeException(elem, m);
	}
	if (!isCID(s)) {
	    String m = "'" + s + "' is an invalid identifier.";
	    throw new NodeException(elem.getAttributeNode(attrName), m);
	}
	return s;
    }
}
